package com.cultural.heritage.controller.wx;


import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.entity.AdvanceOrder;
import com.cultural.heritage.model.entity.Order;
import com.cultural.heritage.model.entity.OrderItems;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.service.book.AdvanceOrderService;
import com.cultural.heritage.service.order.OrderItemService;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.service.user.UserService;
import com.cultural.heritage.service.wx.WeChatService;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import io.swagger.v3.oas.annotations.Hidden;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;

/**
 * 微信小程序相关接口
 **/
@Slf4j
@RestController
@Tag(name = "微信支付接口")
@RequestMapping("/wechat")
@Transactional(rollbackFor = Exception.class)
public class WeChatPayController {

    @Resource
    private UserService userService;

    @Resource
    private OrderService ordersService;

    @Resource
    private WeChatService weChatService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private AdvanceOrderService advanceOrderService;



    /**
     * JSAPI 下单（商品类）
     */
    @PostMapping("/payment/create")
    @Operation(summary = "JSAPI 下单（商品类）", description = "参数：订单id, 权限：所有人, 方法名：createPayment")
    public BaseResponse<PrepayWithRequestPaymentResponse> createPayment(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        String miniOpenId = loginUser.getMiniOpenId();
        ThrowUtils.throwIf(miniOpenId == null, ErrorCode.NOT_FOUND_ERROR, "不是小程序用户");
        Long orderId = commonRequest.getId();
        Order order = ordersService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        ThrowUtils.throwIf(!order.getOrderStatus().equals(OrderStatusConstant.PENDING_PAYMENT), ErrorCode.OPERATION_ERROR, "订单状态错误");
        if (!loginUser.getId().equals(order.getUserId())) {
            throw new BusinessException(ErrorCode.NO_AUTH_ERROR, "你不是该订单用户！");
        }
        PrepayWithRequestPaymentResponse response = weChatService.createPayment(String.valueOf(orderId), miniOpenId, order.getTotalAmount());
        return ResultUtils.success(response);
    }




    /**
     * JSAPI 下单回调（商品类）
     */
    @Hidden
    @PostMapping("/payment/callback")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "JSAPI 下单回调（商品类）", description = "参数：订单id, 权限：所有人, 方法名：callbackPayment")
    public synchronized BaseResponse<Boolean> callbackPayment(HttpServletRequest request) throws IOException {
        // 获取下单信息
        Transaction transaction = weChatService.getTransactionInfo(request);
        System.out.println("下单信息:" + transaction);
        // 支付回调
        boolean result = weChatService.paymentCallback(transaction);
        ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR, "微信支付回调失败");
        return ResultUtils.success(true);
    }





    /**
     * JSAPI 下单（写真预约类）
     */
    @PostMapping("/payment/photo/create")
    @Operation(summary = "JSAPI 下单(写真预约类）", description = "参数：订单id, 权限：所有人, 方法名：createPhotoProductsPayment")
    public BaseResponse<PrepayWithRequestPaymentResponse> createPhotoProductsPayment(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        String miniOpenId = loginUser.getMiniOpenId();
        ThrowUtils.throwIf(miniOpenId == null, ErrorCode.NOT_FOUND_ERROR, "不是小程序用户");
        Long orderId = commonRequest.getId();
        AdvanceOrder advanceOrder = advanceOrderService.getById(orderId);
        ThrowUtils.throwIf(advanceOrder == null, ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        ThrowUtils.throwIf(!advanceOrder.getOrderStatus().equals(OrderStatusConstant.PENDING_PAYMENT), ErrorCode.OPERATION_ERROR, "订单状态错误");
        if (!loginUser.getId().equals(advanceOrder.getUserId())) {
            throw new BusinessException(ErrorCode.NO_AUTH_ERROR, "你不是该订单用户！");
        }
        PrepayWithRequestPaymentResponse response = weChatService.createPhotoProductsPayment(String.valueOf(orderId), miniOpenId, advanceOrder.getTotalAmount());
        return ResultUtils.success(response);
    }



    /**
     * JSAPI 下单回调（写真预约类）
     */
    @Hidden
    @PostMapping("/payment/photo/callback")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "JSAPI 下单回调（写真预约类）", description = "参数：订单id, 权限：所有人, 方法名：callbackPhotoProductsPayment")
    public synchronized BaseResponse<Boolean> callbackPhotoProductsPayment(HttpServletRequest request) throws IOException {
        // 获取下单信息
        Transaction transaction = weChatService.getTransactionInfo(request);
        System.out.println("下单信息:" + transaction);
        // 支付回调
        boolean result = weChatService.paymentPhotoProductsCallback(transaction);
        ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR, "微信支付回调失败");
        return ResultUtils.success(true);
    }




    /**
     * Web端管理员全额退款(商品类）
     * @param commonRequest 订单id
     */
    @PostMapping("/refund/create")
    @Operation(summary = "Web端管理员全额退款(商品类）", description = "参数：订单id, 权限：web端管理员, 方法名：createRefund")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Refund> createRefund(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long orderId = commonRequest.getId();
        Order order = ordersService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        Refund refund = weChatService.refundPayment(String.valueOf(orderId), order.getTotalAmount());
        return ResultUtils.success(refund);
    }



    /**
     * Web端管理员全额退款(写真预约类）
     * @param commonRequest 订单id
     */
    @PostMapping("/refund/photo/create")
    @Operation(summary = "Web端管理员全额退款(写真预约类）", description = "参数：订单id, 权限：web端管理员, 方法名：createPhotoProductsRefund")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Refund> createPhotoProductsRefund(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long orderId = commonRequest.getId();
        AdvanceOrder advanceOrder = advanceOrderService.getById(orderId);
        ThrowUtils.throwIf(advanceOrder == null, ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        Refund refund = weChatService.refundPhotoProductsPayment(String.valueOf(orderId), advanceOrder.getTotalAmount());
        return ResultUtils.success(refund);
    }




    /**
     * Web管理员部分退款
     * @param commonRequest 订单明细id
     */
    @PostMapping("/refund/part/create")
    @Operation(summary = "Web管理员部分退款", description = "参数：订单id, 权限：web端管理员, 方法名：createPartRefund")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Refund> createPartRefund(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long orderItemsId = commonRequest.getId();
        OrderItems orderItems = orderItemService.getById(orderItemsId);
        ThrowUtils.throwIf(orderItems == null, ErrorCode.OPERATION_ERROR, "订单明细不存在");

        Refund refund = weChatService.refundPartPayment(String.valueOf(orderItemsId), orderItems.getItemTotalAmount());
        return ResultUtils.success(refund);
    }




    /**
     * Web管理员剩余部分退款
     * @param commonRequest 订单id
     */
    @Hidden
    @PostMapping("/refund/rest/create")
    @Operation(summary = "Web管理员剩余部分退款", description = "参数：订单id, 权限：web端管理员, 方法名：createRestRefund")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Refund> createRestRefund(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long orderId = commonRequest.getId();
        Order order = ordersService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "订单不存在");

        Refund refund = weChatService.refundRestPayment(String.valueOf(orderId), order.getTotalAmount());
        return ResultUtils.success(refund);
    }





    /**
     * 全额退款回调（商品类）
     */
    @Hidden
    @PostMapping("/refund/callback")
    @Operation(summary = "全额退款回调（商品类）", description = "参数：订单id, 权限：web端管理员, 方法名：callbackRefund")
    public BaseResponse<Boolean> callbackRefund(HttpServletRequest request) {
        // 获取退款信息
        RefundNotification refundNotification = weChatService.getRefundInfo(request);
        // 退款回调
        boolean result = weChatService.refundCallback(refundNotification);
        ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR, "退款回调失败");
        return ResultUtils.success(true);
    }


    /**
     * 全额退款回调（写真预约类）
     */
    @Hidden
    @PostMapping("/refund/photo/callback")
    @Operation(summary = "全额退款回调（写真预约类）", description = "参数：订单id, 权限：web端管理员, 方法名：callbackPhotoProductsRefund")
    public BaseResponse<Boolean> callbackPhotoProductsRefund(HttpServletRequest request) {
        // 获取退款信息
        RefundNotification refundNotification = weChatService.getRefundInfo(request);
        // 退款回调
        boolean result = weChatService.refundPhotoProductsCallback(refundNotification);
        ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR, "退款回调失败");
        return ResultUtils.success(true);
    }




    /**
     * 部分退款回调
     */
    @Hidden
    @PostMapping("/refund/part/callback")
    @Operation(summary = "部分退款回调", description = "参数：订单id, 权限：web端管理员, 方法名：callbackRefundPart")
    public BaseResponse<Boolean> callbackRefundPart(HttpServletRequest request) {
        // 获取退款信息
        RefundNotification refundNotification = weChatService.getRefundInfo(request);
        // 退款回调
        boolean result = weChatService.refundPartCallback(refundNotification);
        ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR, "退款回调失败");
        return ResultUtils.success(true);
    }




    /**
     * 剩余退款回调
     */
    @Hidden
    @PostMapping("/refund/rest/callback")
    @Operation(summary = "剩余退款回调", description = "参数：订单id, 权限：web端管理员, 方法名：callbackRestRefund")
    public BaseResponse<Boolean> callbackRestRefund(HttpServletRequest request) {
        // 获取退款信息
        RefundNotification refundNotification = weChatService.getRefundInfo(request);
        // 退款回调
        boolean result = weChatService.refundRestCallback(refundNotification);
        ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR, "退款回调失败");
        return ResultUtils.success(true);
    }


















//    /**
//     * 发送订阅消息
//     */
//    @GetMapping("/refund/callback")
//    public BaseResponse<Boolean> testSendMessage() {
//        String miniOpenId = "oydCP7TGAWUqN_d0g3y6bbQkbe2I";
//        String templateId = "AJqdU29ETU1kzBGzXaxA-OVs5AvGY1s4xaSJY1JNaiE";
//        boolean subscribeMessage = weChatService.sendSubscribeMessage(miniOpenId, templateId);
//        return ResultUtils.success(subscribeMessage);
//    }

}
