package com.cultural.heritage.service.good.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.CartExperienceMapper;
import com.cultural.heritage.mapper.CartRecordMapper;
import com.cultural.heritage.mapper.GoodMapper;
import com.cultural.heritage.model.dto.good.GoodQueryRequest;
import com.cultural.heritage.model.dto.order.BookingOrderQueryRequest;
import com.cultural.heritage.model.dto.order.BuySingleServiceGoodVerifyRequest;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.order.PendingServiceGoodService;
import com.cultural.heritage.service.order.PendingServiceOrderService;
import com.cultural.heritage.utils.SqlUtils;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class GoodServiceImpl extends ServiceImpl<GoodMapper, Good> implements GoodService {


    @Resource
    private CartRecordMapper cartRecordMapper;


    @Resource
    private CartExperienceMapper cartExperienceMapper;


    @Resource
    private PendingServiceGoodService pendingServiceGoodService;


    @Resource
    private PendingServiceOrderService pendingServiceOrderService;


    /**
     * 商品查询条件
     */
    @Override
    public QueryWrapper<Good> getGoodQueryWrapper(GoodQueryRequest goodQueryRequest, boolean isServiceGood) {
        if (goodQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }
        Long id = goodQueryRequest.getId();
        String name = goodQueryRequest.getName();
        String type = goodQueryRequest.getType();
        String festivalName = goodQueryRequest.getFestivalName();
        Integer isShelves = goodQueryRequest.getIsShelves();
        String sortField = goodQueryRequest.getSortField();
        String sortOrder = goodQueryRequest.getSortOrder();

        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.like(StringUtils.isNotBlank(name), "name", name);
        int isGoodType = 0;
        if (!isServiceGood){
            isGoodType = 1;
            queryWrapper.like(StringUtils.isNotBlank(type), "type", type);
            queryWrapper.eq(StringUtils.isNotBlank(festivalName), "festivalOrder", festivalName);
        }
        queryWrapper.eq(ObjectUtils.isNotEmpty(isShelves), "isShelves", isShelves);
        queryWrapper.eq(ObjectUtils.isNotEmpty(isGoodType), "isGoodType", isGoodType);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }





    /**
     * 校验
     */
    @Override
    public void validGood(Good good, boolean update) {
        Long id = good.getId();
        String name = good.getName();
        String type = good.getType();
        String goodImg = good.getGoodImg();
        String intro = good.getIntro();
        String richText = good.getRichText();
        String label = good.getLabel();
        Integer inventory = good.getInventory();
        String festivalName = good.getFestivalName();
        BigDecimal price = good.getPrice();

        if (update) {
            if (id == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数id错误");
            }
        }
        if (StringUtils.isAnyBlank(type, goodImg, intro, richText, label, name, festivalName)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为空");
        }
        if (ObjectUtils.isEmpty(inventory) || inventory < 1) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "库存量不能小于1");
        }
        if (ObjectUtils.isEmpty(price) || price.compareTo(BigDecimal.ZERO) < 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "商品价格不能小于等于0");
        }


    }


    /**
     * 更新常规类购物车中的小计
     */
    @Override
    public List<CartRecord> updateCartGoodPrice(Long goodId, BigDecimal targetPrice) {
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("goodId", goodId);
        List<CartRecord> cartRecordList = cartRecordMapper.selectList(queryWrapper);
        // 更新购物车中的小计
        for (CartRecord cartRecord : cartRecordList) {
            Integer quantity = cartRecord.getQuantity();
            cartRecord.setSubtotal(targetPrice.multiply(BigDecimal.valueOf(quantity)));
        }
        return cartRecordList;
    }



    /**
     * 更新服务类购物车中的小计
     */
    @Override
    public List<CartExperience> updateCartExperienceGoodPrice(Long goodId, BigDecimal targetPrice) {
        QueryWrapper<CartExperience> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("goodId", goodId);
        List<CartExperience> cartExperienceList = cartExperienceMapper.selectList(queryWrapper);
        // 更新购物车中的小计
        for (CartExperience cartExperience : cartExperienceList) {
            Integer quantity = cartExperience.getQuantity();
            cartExperience.setSubtotal(targetPrice.multiply(BigDecimal.valueOf(quantity)));
        }
        return cartExperienceList;
    }



    /**
     * 判断商品是否存在或者下架
     */
    @Override
    public void checkGoodIsExist(Good good) {
        ThrowUtils.throwIf(good == null || good.getIsShelves() == 0, ErrorCode.OPERATION_ERROR, "商品不存在或者已下架");
    }


    /**
     * 判断常规类商品库存是否充足
     */
    @Override
    public void checkInventoryIsEnough(Integer neededQuantity, Integer actualQuantity) {
        ThrowUtils.throwIf(neededQuantity > actualQuantity, ErrorCode.OPERATION_ERROR, "商品库存不足");
    }




    /**
     * 封装一个Map集合（键：BookingOrderQueryRequest.toString()，值：服务类商品待处理商品记录）
     */
    @Override
    public Map<String, PendingServiceGood> getPendingServiceGoodMap() {
        Map<String, PendingServiceGood> map = new HashMap<>();
        List<PendingServiceGood> pendingServiceGoodList = pendingServiceGoodService.list();
        for (PendingServiceGood pendingServiceGood : pendingServiceGoodList) {
            Long goodId = pendingServiceGood.getGoodId();
            String reservationDate = pendingServiceGood.getReservationDate();
            String timeSlot = pendingServiceGood.getTimeSlot();
            BookingOrderQueryRequest bookingOrderQueryRequest = new BookingOrderQueryRequest(goodId, reservationDate, timeSlot);
            map.put(bookingOrderQueryRequest.toString(), pendingServiceGood);
        }
        return map;
    }



    /**
     * 校验购买的服务类商品预约情况是否合理
     */
    @Override
    public void checkServiceGoodBookingDetailIsProperly(Long goodId, String reservationDate, String timeSlot, Integer quantity) {
        BookingOrderQueryRequest bookingOrderQueryRequest = new BookingOrderQueryRequest(goodId, reservationDate, timeSlot);
        Map<String, PendingServiceGood> pendingServiceGoodMap = this.getPendingServiceGoodMap();
        PendingServiceGood pendingServiceGood = pendingServiceGoodMap.get(bookingOrderQueryRequest.toString());

        Integer isAvailable = pendingServiceGood.getIsAvailable();
        ThrowUtils.throwIf(isAvailable == 0, ErrorCode.OPERATION_ERROR, "当前时间段不可预约");

        String startTime = timeSlot.split("-")[0];
        String advanceDateTimeStr = reservationDate + " " + startTime + ":00";
        String currentDateTimeStr = DateUtil.now();

        Date advanceDateTime = DateUtil.parse(advanceDateTimeStr, "yyyy-MM-dd HH:mm:ss");
        Date currentDateTime = DateUtil.parse(currentDateTimeStr, "yyyy-MM-dd HH:mm:ss");

        int result = DateUtil.compare(advanceDateTime, currentDateTime);
        ThrowUtils.throwIf(result <= 0, ErrorCode.OPERATION_ERROR, "当前时间段已过期");

        Long pendingId = pendingServiceGood.getId();
        QueryWrapper<PendingServiceOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pendingId", pendingId)
                        .and(wrapper -> wrapper.eq("orderItemStatus", OrderStatusConstant.PENDING_SHIPMENT).or().eq("orderItemStatus", OrderStatusConstant.PENDING_PAYMENT));
        List<PendingServiceOrder> pendingServiceOrderList = pendingServiceOrderService.list(queryWrapper);
        Integer totalNumber = 0;
        for (PendingServiceOrder pendingServiceOrder : pendingServiceOrderList) {
            totalNumber += pendingServiceOrder.getQuantity();
        }
        Integer maxNumber = pendingServiceGood.getMaxNumber();
        ThrowUtils.throwIf(totalNumber + quantity > maxNumber, ErrorCode.OPERATION_ERROR, "当前预约数量已超过上限");
    }




    /**
     * 校验购买的服务类购物车中的商品预约情况是否合理
     */
    @Override
    public boolean checkServiceGoodCartBookingDetailIsProperly(Long goodId, String reservationDate, String timeSlot, Integer quantity, Map<String, PendingServiceGood> pendingServiceGoodMap, List<PendingServiceOrder> pendingServiceOrderList) {
        BookingOrderQueryRequest bookingOrderQueryRequest = new BookingOrderQueryRequest(goodId, reservationDate, timeSlot);
        PendingServiceGood pendingServiceGood = pendingServiceGoodMap.get(bookingOrderQueryRequest.toString());

        Integer isAvailable = pendingServiceGood.getIsAvailable();
        if (isAvailable == 0) return false;

        String startTime = timeSlot.split("-")[0];
        String advanceDateTimeStr = reservationDate + " " + startTime + ":00";
        String currentDateTimeStr = DateUtil.now();

        Date advanceDateTime = DateUtil.parse(advanceDateTimeStr, "yyyy-MM-dd HH:mm:ss");
        Date currentDateTime = DateUtil.parse(currentDateTimeStr, "yyyy-MM-dd HH:mm:ss");

        int result = DateUtil.compare(advanceDateTime, currentDateTime);
        if (result <= 0) return false;

        Long pendingId = pendingServiceGood.getId();
        pendingServiceOrderList = pendingServiceOrderList.stream().filter(pendingServiceOrder -> {
            String orderItemStatus = pendingServiceOrder.getOrderItemStatus();
            Long serviceOrderPendingId = pendingServiceOrder.getPendingId();
            return (orderItemStatus.equals(OrderStatusConstant.PENDING_SHIPMENT) || orderItemStatus.equals(OrderStatusConstant.PENDING_PAYMENT))
            && serviceOrderPendingId.equals(pendingId);
        }).toList();

        Integer totalNumber = 0;
        for (PendingServiceOrder pendingServiceOrder : pendingServiceOrderList) {
            totalNumber += pendingServiceOrder.getQuantity();
        }
        Integer maxNumber = pendingServiceGood.getMaxNumber();
        return totalNumber + quantity <= maxNumber;
    }



    /**
     * 校验服务类商品预约参数
     */
    @Override
    public void validServiceGoodBookingParameter(BuySingleServiceGoodVerifyRequest buySingleServiceGoodVerifyRequest) {
        Long goodId = buySingleServiceGoodVerifyRequest.getGoodId();
        String reservationDate = buySingleServiceGoodVerifyRequest.getReservationDate();
        String timeSlot = buySingleServiceGoodVerifyRequest.getTimeSlot();
        Integer quantity = buySingleServiceGoodVerifyRequest.getQuantity();
        ThrowUtils.throwIf(goodId == null, ErrorCode.OPERATION_ERROR, "商品id为null");
        ThrowUtils.throwIf(StringUtils.isAnyBlank(reservationDate, timeSlot), ErrorCode.OPERATION_ERROR, "预约日期或预约时间段为空");
        ThrowUtils.throwIf(quantity <= 0, ErrorCode.OPERATION_ERROR, "商品数量小于等于0");
    }


}
